export declare class Collections {
    private readonly firestore;
    private readonly name;
    constructor(firestore: FirestoreLike, name: CollectionNameStr);
    getOrCreate<T>(batch: WriteBatchLike, documentReference: DocumentReferenceLike, createRecord: () => T): Promise<GetOrCreateRecord<T>>;
    set<T>(id: string, value: T): Promise<void>;
    get<T>(id: string): Promise<T | undefined>;
    getByID<T>(id: string): Promise<T | undefined>;
    getByFieldValue<T>(field: string, value: ValueType): Promise<T | undefined>;
    getByFieldValues<T>(clauses: ReadonlyArray<Clause>): Promise<T | undefined>;
    private first;
    listByFieldValue<T>(field: string, value: ValueType): Promise<ReadonlyArray<T>>;
    private createQuery;
    private snapshotToRecords;
    collection(): CollectionReferenceLike;
    list<T>(clauses: ReadonlyArray<Clause>, opts?: ListOpts): Promise<ReadonlyArray<T>>;
    iterate<T>(clauses: ReadonlyArray<Clause>, opts?: IterateOpts): Promise<Cursor<T>>;
    deleteByID(batch: WriteBatchLike, provider: () => Promise<ReadonlyArray<IDRecord>>): Promise<void>;
}
export interface FirestoreProvider {
    (): FirestoreLike;
}
export interface Cursor<T> {
    hasNext(): boolean;
    next(): Promise<ReadonlyArray<T>>;
}
export interface IterateOpts {
    readonly limit?: number;
    readonly offset?: number;
    readonly startAfter?: any[];
    readonly startAt?: any[];
    readonly orderBy?: ReadonlyArray<OrderByClause>;
}
export declare type OrderByClause = [string, OrderByDirectionLike | undefined];
export interface ListOpts extends IterateOpts {
}
export declare type ValueType = object | string | number;
export declare type Clause = [string, WhereFilterOpLike, ValueType];
export declare class Clauses {
    static assertPresent(clause: Clause): void;
    static fields(clauses: ReadonlyArray<Clause>): string[];
    static values(clauses: ReadonlyArray<Clause>): ValueType[];
}
export interface IDRecord {
    readonly id: string;
}
export interface GetOrCreateRecord<T> {
    readonly created: boolean;
    readonly record: T;
}
export interface FirestoreLike {
    collection(collectionPath: string): CollectionReferenceLike;
}
export interface WriteBatchLike {
    create(documentRef: DocumentReferenceLike, data: DocumentDataLike): WriteBatchLike;
    delete(documentRef: DocumentReferenceLike): WriteBatchLike;
}
export interface DocumentReferenceLike {
    get(): Promise<DocumentSnapshotLike>;
    set(data: DocumentDataLike, options?: SetOptionsLike): Promise<any>;
}
export interface SetOptionsLike {
    readonly merge?: boolean;
}
export interface WriteResultLike {
    readonly writeTime: TimestampLike;
}
export interface TimestampLike {
}
export interface DocumentSnapshotLike {
    readonly exists: boolean;
    data(): DocumentDataLike | undefined;
}
export interface CollectionReferenceLike {
    doc(documentPath: string): DocumentReferenceLike;
    where(fieldPath: string, opStr: WhereFilterOpLike, value: any): QueryLike;
    limit(size: number): QueryLike;
}
export declare type DocumentDataLike = {
    [field: string]: any;
};
export interface QueryLike {
    get(): Promise<QuerySnapshotLike>;
    where(fieldPath: string, opStr: WhereFilterOpLike, value: any): QueryLike;
    limit(limit: number): QueryLike;
    offset(offset: number): QueryLike;
    orderBy(fieldPath: string, directionStr?: OrderByDirectionLike): QueryLike;
    startAt(...fieldValues: any[]): QueryLike;
    startAfter(...fieldValues: any[]): QueryLike;
    endAt(...fieldValues: any[]): QueryLike;
    endBefore(...fieldValues: any[]): QueryLike;
}
export declare type WhereFilterOpLike = '<' | '<=' | '==' | '>=' | '>' | 'array-contains';
export declare type OrderByDirectionLike = 'desc' | 'asc';
export interface QuerySnapshotLike {
    readonly docs: QueryDocumentSnapshotLike[];
}
export interface QueryDocumentSnapshotLike {
    data(): DocumentDataLike;
    get(fieldPath: string): any;
}
export declare type UserIDStr = string;
export declare type CollectionNameStr = string;
export declare type FirestorePrimitive = string | number | boolean | null;
export interface FirestoreTypedArray<T extends FirestorePrimitive | FirestoreDict | FirebaseDictTyped<T>> {
    readonly [id: number]: T;
}
export interface FirestoreArray {
    readonly [id: number]: FirestoreDict | FirestoreArray | FirestorePrimitive;
}
export declare type FirebaseDictTyped<T> = {
    readonly [P in keyof T]: FirestorePrimitive;
};
export interface FirestoreDict {
    readonly [id: string]: FirestoreDict | FirestoreArray | FirestorePrimitive | FirebaseDictTyped<any>;
}
export declare type FirestoreValue = FirestoreDict | FirestoreArray | FirestorePrimitive;
